﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  public abstract class AddNewColumnCommandBase : ColumnGroupCommandBase {
    private AddNewColumnCommandBase()
      : base(null, string.Empty) {
    }

    protected AddNewColumnCommandBase(DataSet dataSet, string columnGroupName, string columnName, Type columnType)
      : base(dataSet, columnGroupName) {
      this.ColumnName = columnName;
      if (!columnType.IsSubclassOf(typeof(ColumnBase)) || columnType == null)
        throw new CommandExecutionException("Added Column must be a subtype of ColumnBase", this);
      this.ColumnType = columnType;
    }

    private ColumnBase newColumn;
    protected ColumnBase NewColumn {
      get { return this.newColumn; }
      set { this.newColumn = value; }
    }

    [Storable]
    private string columnName;
    protected string ColumnName {
      get { return this.columnName; }
      set { this.columnName = value; }
    }

    [Storable]
    private Type columnType;
    protected Type ColumnType {
      get { return this.columnType; }
      set { this.columnType = value; }
    }

    public override void Execute() {
      base.Execute();
      newColumn = (ColumnBase)Activator.CreateInstance(ColumnType, new object[] { this.ColumnName });
      newColumn.Resize(ColumnGroup.RowCount);
      ColumnGroup.AddColumn(NewColumn);
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ColumnGroup.RemoveColumn(NewColumn);
      newColumn = null;
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
