﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.Model;
using System.Windows.Forms.DataVisualization.Charting;
using System.Drawing;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfo("Create time series columns", 1, ColumnGroupState.ColumnSelected, "Column Commands",
    Position = 16, SelectedColumns = 1)]
  public class CreateTimeSeriesColumnsCommand : ColumnCommandBase {
    private int addedColumnsCount;
    private const double SETVALUE = 1.0;
    private const double NOTSETVALUE = 0.0;

    private CreateTimeSeriesColumnsCommand()
      : base(null, string.Empty, -1) {
    }

    public CreateTimeSeriesColumnsCommand(DataSet dataSet, string columnGroupName, int columnIndex)
      : base(dataSet, columnGroupName, columnIndex) {
    }

    public override string Description {
      get { return "Create time series columns"; }
    }

    public override void Execute() {
      base.Execute();

      //if (Column.NullValuesCount != 0)
      //  throw new CommandExecutionException("Categorial column must not contain null values.", this);

      //int columnIndex = this.ColumnGroup.IndexOfColumn(Column);
      //Dictionary<string, DoubleColumn> addedColumns = new Dictionary<string, DoubleColumn>();
      //foreach (string value in Column.ValuesEnumerable.Cast<IComparable>().Distinct().OrderBy(c => c).Select(o => o.ToString())) {
      //  DoubleColumn doubleColumn = new DoubleColumn(Column.Name + " " + value.ToString());
      //  addedColumns[value] = doubleColumn;
      //  ColumnGroup.InsertColumn(columnIndex + addedColumnsCount + 1, doubleColumn);
      //  addedColumnsCount++;
      //}

      //for (int row = 0; row < ColumnGroup.RowCount; row++) {
      //  string value = Column.GetValue(row).ToString();
      //  foreach (KeyValuePair<string, DoubleColumn> pair in addedColumns) {
      //    if (pair.Key == value)
      //      pair.Value.AddValue(SETVALUE);
      //    else
      //      pair.Value.AddValue(NOTSETVALUE);
      //  }
      //}

      // Set Formula Name
      Chart chart1 = new Chart();
      string formulaName = "DetrendedPriceOscillator";

      FinancialFormula formula = (FinancialFormula)Enum.Parse(typeof(FinancialFormula), formulaName, true);
      
      // Formulas with one input value
      if (formulaName == "DetrendedPriceOscillator" || formulaName == "MovingAverageConvergenceDivergence" || formulaName == "Performance" || formulaName == "RateOfChange"
      || formulaName == "TripleExponentialMovingAverage") {
        chart1.DataManipulator.FinancialFormula(formula, "10", "Input:Y4", "Indicators");
      }

      // Relative Strength Index
      else if (formulaName == "RelativeStrengthIndex") {

        chart1.DataManipulator.FinancialFormula(formula, "10", "Input:Y4", "Indicators");

        // Set minimum and maximum for Y axis
        chart1.ChartAreas["Indicator"].AxisY.Minimum = 0;
        chart1.ChartAreas["Indicator"].AxisY.Maximum = 100;

        // Create strip lines used with Relative strength index.
        StripLine stripLine = new StripLine();
        chart1.ChartAreas["Indicator"].AxisY.StripLines.Add(stripLine);
        stripLine.Interval = 70;
        stripLine.StripWidth = 30;
        stripLine.BackColor = Color.FromArgb(128, 255, 255, 0);
        stripLine.BorderColor = Color.Teal;

      }

      // Williams %R
      else if (formulaName == "WilliamsR") {

        chart1.DataManipulator.FinancialFormula(formula, "Input:Y,Input:Y2,Input:Y4", "Indicators");

        // Set minimum and maximum for Y axis
        chart1.ChartAreas["Indicator"].AxisY.Minimum = -100;
        chart1.ChartAreas["Indicator"].AxisY.Maximum = 0;

        // Create strip lines used with Williams %R index.
        StripLine stripLine = new StripLine();
        chart1.ChartAreas["Indicator"].AxisY.StripLines.Add(stripLine);
        stripLine.Interval = 80;
        stripLine.StripWidth = 20;
        stripLine.BackColor = Color.FromArgb(128, 255, 255, 0);
        stripLine.BorderColor = Color.Teal;

      }

      // Formulas with two input value
      else if (formulaName == "MassIndex" || formulaName == "VolatilityChaikins" || formulaName == "Performance") {
        chart1.DataManipulator.FinancialFormula(formula, "20", "Input:Y,Input:Y2", "Indicators");
      }

      // Standard deviation
      else if (formulaName == "StandardDeviation") {
        chart1.DataManipulator.FinancialFormula(formula, "15", "Input:Y4", "Indicators");
      }

      // StochasticIndicator
      else if (formulaName == "StochasticIndicator") {

        chart1.DataManipulator.FinancialFormula(formula, "15", "Input:Y,Input:Y2,Input:Y4", "Indicators,SMA");

        // Set attributes for Simple moving average series.
        chart1.Series["SMA"].ChartType = SeriesChartType.Line;
        chart1.Series["SMA"].Color = Color.Yellow;
        chart1.Series["SMA"].ChartArea = "Indicator";
        chart1.Series["SMA"].BorderWidth = 2;

      }
        // All other formulas.
      else {
        chart1.DataManipulator.FinancialFormula(formula, "Input:Y,Input:Y2,Input:Y4", "Indicators");
      }

      // Set minimum for X axis
      chart1.ChartAreas["Indicator"].AxisX.Minimum = DateTime.Parse("1/1/2002").ToOADate();
   

      ColumnGroup.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      int columnIndex = this.ColumnGroup.IndexOfColumn(Column);
      for (int i = 0; i < addedColumnsCount; i++)
        this.ColumnGroup.RemoveColumn(columnIndex + 1);
      addedColumnsCount = 0;
      ColumnGroup.FireChanged();
    }
  }
}
